@@ -23,7 +23,6 @@
 #include "gdb_obstack.h"
 #include "symtab.h"
 #include "symfile.h"
-#include "gdb_assert.h"
 #include "block.h"
 #include "objfiles.h"
 #include "gdbtypes.h"
@@ -219,14 +218,7 @@ cp_is_anonymous (const char *namespace)
    names.  This makes sure that names get looked for in all namespaces
    that are in scope.  NAME is the natural name of the symbol that
    we're looking for, BLOCK is the block that we're searching within,
-   DOMAIN says what kind of symbols we're looking for, and if SYMTAB
-   is non-NULL, we should store the symtab where we found the symbol
-   in it.
-
-   Class, union, and enum tag names may be used in C++ without specifying
-   class-key or enum.  If searching for a VAR_DOMAIN symbol fails,
-   this function will search STRUCT_DOMAIN.  [This is actually done in
-   cp_lookup_symbol_in_namespace.]  */
+   DOMAIN says what kind of symbols we're looking for.  */
 
 struct symbol *
 cp_lookup_symbol_nonlocal (const char *name,
@@ -247,10 +239,7 @@ cp_lookup_symbol_nonlocal (const char *name,
 
 /* Look up NAME in the C++ namespace NAMESPACE.  Other arguments are
    as in cp_lookup_symbol_nonlocal.  If SEARCH is non-zero, search
-   through base classes for a matching symbol.
-
-   If DOMAIN is VAR_DOMAIN and no matching symbol exists in that domain,
-   this function will search STRUCT_DOMAIN for a match.  */
+   through base classes for a matching symbol.  */
 
 static struct symbol *
 cp_lookup_symbol_in_namespace (const char *namespace,
@@ -260,30 +249,18 @@ cp_lookup_symbol_in_namespace (const char *namespace,
 {
   if (namespace[0] == '\0')
     {
-      struct symbol *sym = lookup_symbol_file (name, block, domain, 0, search);
-
-      if (sym == NULL && domain == VAR_DOMAIN)
-	sym = lookup_symbol_file (name, block, STRUCT_DOMAIN, 0, search);
-
-      return sym;
+      return lookup_symbol_file (name, block, domain, 0, search);
     }
   else
     {
-      struct symbol *sym;
       char *concatenated_name = alloca (strlen (namespace) + 2
 					+ strlen (name) + 1);
 
       strcpy (concatenated_name, namespace);
       strcat (concatenated_name, "::");
       strcat (concatenated_name, name);
-      sym = lookup_symbol_file (concatenated_name, block, domain,
-				cp_is_anonymous (namespace), search);
-
-      if (sym == NULL && domain == VAR_DOMAIN)
-	sym = lookup_symbol_file (concatenated_name, block, STRUCT_DOMAIN,
-				  cp_is_anonymous (namespace), search);
-
-      return sym;
+      return lookup_symbol_file (concatenated_name, block, domain,
+				 cp_is_anonymous (namespace), search);
     }
 }
 
@@ -494,7 +471,7 @@ cp_lookup_symbol_imports_or_template (const char *scope,
 	  struct cleanup *cleanups = make_cleanup (xfree, name_copy);
 	  const struct language_defn *lang = language_def (language_cplus);
 	  struct gdbarch *arch
-	    = get_objfile_arch (SYMBOL_SYMTAB (function)->objfile);
+	    = get_objfile_arch (SYMBOL_OBJFILE (function));
 	  const struct block *parent = BLOCK_SUPERBLOCK (block);
 
 	  while (1)
@@ -536,12 +513,7 @@ cp_lookup_symbol_imports_or_template (const char *scope,
  /* Searches for NAME in the current namespace, and by applying
     relevant import statements belonging to BLOCK and its parents.
     SCOPE is the namespace scope of the context in which the search is
-    being evaluated.
-
-    Class, union, and enum tag names may be used in C++ without specifying
-    class-key or enum.  If searching for a VAR_DOMAIN symbol fails,
-    this function will search STRUCT_DOMAIN.  [This is done in
-    cp_lookup_symbol_in_namespace.]  */
+    being evaluated.  */
 
 struct symbol*
 cp_lookup_symbol_namespace (const char *scope,
@@ -641,7 +613,7 @@ lookup_symbol_file (const char *name,
 {
   struct symbol *sym = NULL;
 
-  sym = lookup_symbol_static (name, block, domain);
+  sym = lookup_symbol_in_static_block (name, block, domain);
   if (sym != NULL)
     return sym;
 
@@ -654,11 +626,11 @@ lookup_symbol_file (const char *name,
       const struct block *global_block = block_global_block (block);
       
       if (global_block != NULL)
-	sym = lookup_symbol_aux_block (name, global_block, domain);
+	sym = lookup_symbol_in_block (name, global_block, domain);
     }
   else
     {
-      sym = lookup_symbol_global (name, block, domain);
+      sym = lookup_global_symbol (name, block, domain);
     }
 
   if (sym != NULL)
@@ -718,7 +690,7 @@ lookup_symbol_file (const char *name,
 
       /* Lookup a class named KLASS.  If none is found, there is nothing
 	 more that can be done.  */
-      klass_sym = lookup_symbol_global (klass, block, STRUCT_DOMAIN);
+      klass_sym = lookup_global_symbol (klass, block, domain);
       if (klass_sym == NULL)
 	{
 	  do_cleanups (cleanup);
@@ -797,14 +769,15 @@ find_symbol_in_baseclass (struct type *parent_type, const char *name,
       len = strlen (base_name) + 2 + strlen (name) + 1;
       concatenated_name = xrealloc (concatenated_name, len);
       xsnprintf (concatenated_name, len, "%s::%s", base_name, name);
-      sym = lookup_symbol_static (concatenated_name, block, VAR_DOMAIN);
+      sym = lookup_symbol_in_static_block (concatenated_name, block,
+					   VAR_DOMAIN);
       if (sym != NULL)
 	break;
 
       /* Nope.  We now have to search all static blocks in all objfiles,
 	 even if block != NULL, because there's no guarantees as to which
 	 symtab the symbol we want is in.  */
-      sym = lookup_static_symbol_aux (concatenated_name, VAR_DOMAIN);
+      sym = lookup_static_symbol (concatenated_name, VAR_DOMAIN);
       if (sym != NULL)
 	break;
 
@@ -875,7 +848,7 @@ cp_lookup_nested_symbol (struct type *parent_type,
 	concatenated_name = alloca (size);
 	xsnprintf (concatenated_name, size, "%s::%s",
 		 parent_name, nested_name);
-	sym = lookup_static_symbol_aux (concatenated_name, VAR_DOMAIN);
+	sym = lookup_static_symbol (concatenated_name, VAR_DOMAIN);
 	if (sym != NULL)
 	  return sym;
 
