@@ -31,12 +31,9 @@
 #endif
 
 #include <sys/types.h>
-#include <string.h>
-#include <errno.h>
 #include <sys/time.h>
-#include "exceptions.h"
-#include "gdb_assert.h"
 #include "gdb_select.h"
+#include "observer.h"
 
 /* Tell create_file_handler what events we are interested in.
    This is used by the select version of the event loop.  */
@@ -441,10 +438,7 @@ start_event_loop (void)
 	  /* If we long-jumped out of do_one_event, we probably didn't
 	     get around to resetting the prompt, which leaves readline
 	     in a messed-up state.  Reset it here.  */
-	  /* FIXME: this should really be a call to a hook that is
-	     interface specific, because interfaces can display the
-	     prompt in their own way.  */
-	  display_gdb_prompt (0);
+	  observer_notify_command_error ();
 	  /* This call looks bizarre, but it is required.  If the user
 	     entered a command that caused an error,
 	     after_char_processing_hook won't be called from
