@@ -1,6 +1,6 @@
 /* OS ABI variant handling for GDB.
 
-   Copyright (C) 2001-2013 Free Software Foundation, Inc.
+   Copyright (C) 2001-2014 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -19,9 +19,6 @@
 
 #include "defs.h"
 
-#include "gdb_assert.h"
-#include <string.h>
-
 #include "osabi.h"
 #include "arch-utils.h"
 #include "gdbcmd.h"
@@ -53,7 +50,6 @@ static const char * const gdb_osabi_names[] =
   "SVR4",
   "GNU/Hurd",
   "Solaris",
-  "OSF/1",
   "GNU/Linux",
   "FreeBSD a.out",
   "FreeBSD ELF",
